/* main.c
 * Copyright (C) 2007-2012 Linutop SAS <http://www.linutop.com/>
 * This file is part of the linutop config editor
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gio/gio.h>
#include <unique/unique.h>

#include "main.h"

#define LOCKCFG "/cdrom/lock.cfg"

enum {
    SECU_HOME = 0,
    SECU_FULL,
    SECU_LOCK
};

guint current_secu;
gboolean have_to_reboot = FALSE;

/* callbacks */
static void
run_help (GtkWidget *widget, gpointer data)
{
    run_user_cmd ("yelp ghelp:linutopos", TRUE);
}

static void
delete_event (GtkWidget *widget, GdkEvent *event, gpointer data)
{
    /* restore the initial kbd conf */
    restore_kbd_config ();
    gtk_main_quit ();
}

static void
kbd_btn_clicked (GtkWidget *widget, gpointer data)
{
    layout_dialog_run ();
}

static void
tz_btn_clicked (GtkWidget *widget, gpointer data)
{
    tz_dialog_run ();
}

static void
locale_btn_clicked (GtkWidget *widget, gpointer data)
{
    locale_dialog_run ();
}

static void
display_btn_clicked (GtkWidget *widget, gpointer data)
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->auto_xorg_chk)))
        run_user_cmd ("gnome-display-properties", TRUE);
    else
        display_dialog_run ();
}

static void
on_restore_clicked (GtkButton *btn, gpointer data)
{
    restore_dialog_run ();
}

static void
on_win_share_btn_clicked (GtkButton *btn, gpointer data)
{
    gtk_widget_show (dialog->mount_dialog);
}

static void
on_nm_check_toggled (GtkWidget *widget, gpointer data)
{
    gtk_widget_set_sensitive (GTK_WIDGET (data),
        !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)));
}

static void
change_lock_setup (GtkWidget *w, gpointer data)
{
    xfce_exec ("locksetup", FALSE, TRUE, NULL);
}

static void
change_pwd (GtkWidget *w, gpointer data)
{
    gchar *tmp = get_lock_passwd ();
    if (tmp)
    {
        g_free (dialog->lock_pwd);
        dialog->lock_pwd = tmp;
    }
}

static void
security_toggled (GtkWidget *btn, gpointer data)
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->full_chk)))
    {
        current_secu = SECU_FULL;
        return;
    }

    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->home_chk)))
    {
        current_secu = SECU_HOME;
        return;
    }
}

static void
lock_toggled (GtkWidget *btn, gpointer data)
{
    gchar *tmp = NULL;
    if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (btn)))
    {
        gtk_widget_set_sensitive (dialog->lock_pwd_btn, FALSE);
        return;
    }

    gtk_widget_set_sensitive (dialog->lock_pwd_btn, TRUE);
    if (!dialog->lock_pwd)
    {
        if ((tmp = get_lock_passwd ()) == NULL)
        {
            switch (current_secu){
                case SECU_HOME:
                    gtk_toggle_button_set_active (
                        GTK_TOGGLE_BUTTON (dialog->home_chk), TRUE);
                    break;
                case SECU_FULL:
                    gtk_toggle_button_set_active (
                        GTK_TOGGLE_BUTTON (dialog->full_chk), TRUE);
                    break;
            }
            return;
        }

        dialog->lock_pwd = tmp;
        current_secu = SECU_LOCK;
    }

    /* run the locksetup if no config already exists */
    if (g_access (LOCKCFG, F_OK) != 0)
    {
        g_spawn_command_line_async ("locksetup", NULL);
    }
}

static void
save_config (GtkWidget *widget, gpointer data)
{
    const gchar *timezone = get_tz_value ();
    char *command =
        g_strconcat ("dbus-send --system --dest=org.gnome.ClockApplet.Mechanism"
                     " / org.gnome.ClockApplet.Mechanism.SetTimezone "
                     "string:/usr/share/zoneinfo/",
                     timezone, NULL);
    _system (command);
    g_free (command);

    mount_save ();
    run_user_cmd ("linutop-startup --samba", FALSE);

#ifndef WITH_DEMO
    write_config ();

    if (get_reboot_needed ())
    {
        /* destroy the main window */
        gtk_widget_destroy (dialog->dialog);
        have_to_reboot = warn_about_changes ();
    }
#endif /* WITH_DEMO */

    gtk_main_quit ();
}

static void
set_autostarted_app_btn ()
{
    gchar *auto_cmd = NULL;
    gchar *auto_icon = NULL;

    if (get_autostart_infos (&auto_cmd, &auto_icon))
    {
        dialog->autostarted_btn = new_run_cmd_button (auto_icon, auto_cmd,
                                        "appstarter", TRUE);
        g_free (auto_cmd);
        g_free (auto_icon);
    }
    else
        dialog->autostarted_btn = new_run_cmd_button ("xfce4-autostart-editor",
                                        _("Configure autostarted applications"),
                                        "appstarter", TRUE);
}

static void
on_appstarterrc_changed (GFileMonitor *monitor,
                         GFile *file,
                         GFile *other_file,
                         GFileMonitorEvent event,
                         gpointer data)
{
    gchar *auto_cmd = NULL;
    gchar *auto_icon = NULL;
    GtkWidget *img;

    switch (event)
    {
        case G_FILE_MONITOR_EVENT_CHANGED:
        case G_FILE_MONITOR_EVENT_CREATED:
            if (!get_autostart_infos (&auto_cmd, &auto_icon))
                break;
            img = get_image_from_str (auto_icon);
            if (img)
                gtk_button_set_image (GTK_BUTTON (dialog->autostarted_btn), img);
            gtk_button_set_label (GTK_BUTTON (dialog->autostarted_btn), auto_cmd);
            break;

        default:
            break;
    }
}

static void
create_dialog ()
{
    GtkWidget *label, *table, *vbox, *hbox, *notebook, *vbox2;
    GtkWidget *backup_btn, *format_btn, *restore_btn, *image, *frame;
    GtkWidget *net_btn, *display_btn;
    GtkWidget *advanced_vbox, *security_vbox;
    GtkWidget *win_share_btn, *shared_folders_btn;
    GtkWidget *img, *hr;
    GtkWidget *tools, *about, *aboutsysinfo;
    GtkWidget *vnc_btn;
    GtkWidget *nm_vbox, *nm_hbox;
    GtkSizeGroup *size_group;
    gboolean nm_is_there = FALSE;

    gtk_window_set_default_icon_name ("preferences-desktop");

    dialog->dialog = gtk_dialog_new ();
    gtk_window_set_title (GTK_WINDOW (dialog->dialog), _("Linutop Configuration Panel"));
    gtk_window_set_resizable (GTK_WINDOW (dialog->dialog), FALSE);
    gtk_window_set_position (GTK_WINDOW (dialog->dialog), GTK_WIN_POS_CENTER);

    notebook = gtk_notebook_new ();
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog->dialog)->vbox),
              notebook);

    /*
     * General tab
     */
    vbox = gtk_vbox_new (FALSE, 0);

    size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);

    label = gtk_label_new (_("General"));
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);

    table = gtk_table_new (2, 2, FALSE);

    /* local settings */
    frame = xfce_framebox_new (_("Local settings"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), table);
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 4);

    label = gtk_label_new (_("Language: "));
    gtk_size_group_add_widget (size_group, label);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL,
             GTK_SHRINK, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    DEMO_SET_SENSITIVE (label);
    dialog->locale_btn = gtk_button_new ();
    gtk_table_attach (GTK_TABLE (table), dialog->locale_btn, 1, 2, 0, 1,
             GTK_FILL | GTK_EXPAND, GTK_SHRINK, 0, 0);
    DEMO_SET_SENSITIVE (dialog->locale_btn);

    label = gtk_label_new (_("Keyboard: "));
    gtk_size_group_add_widget (size_group, label);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL,
             GTK_SHRINK, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    dialog->kbd_btn = gtk_button_new ();
    gtk_table_attach (GTK_TABLE (table), dialog->kbd_btn, 1, 2, 1, 2,
             GTK_FILL | GTK_EXPAND, GTK_SHRINK, 0, 0);

    label = gtk_label_new (_("Timezone: "));
    gtk_size_group_add_widget (size_group, label);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL,
             GTK_SHRINK, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    dialog->tz_btn = gtk_button_new ();
    gtk_table_attach (GTK_TABLE (table), dialog->tz_btn, 1, 2, 2, 3,
             GTK_FILL | GTK_EXPAND, GTK_SHRINK, 0, 0);

    /* Display settings */
    hbox = gtk_hbox_new (FALSE, 0);

    dialog->auto_xorg_chk = gtk_check_button_new_with_label (_(
                "Automatic: "));
    gtk_size_group_add_widget (size_group, dialog->auto_xorg_chk);
    gtk_box_pack_start (GTK_BOX (hbox), dialog->auto_xorg_chk,
                        FALSE, FALSE, 0);

    display_btn = gtk_button_new_with_label (_("Screen configuration"));
    gtk_button_set_image (GTK_BUTTON (display_btn),
                          gtk_image_new_from_icon_name (
                                        "gnome-display-properties",
                                        GTK_ICON_SIZE_BUTTON));
    gtk_button_set_alignment (GTK_BUTTON (display_btn), 0, 0.5);
    gtk_box_pack_start (GTK_BOX (hbox), display_btn, TRUE, TRUE, 0);

    frame = xfce_framebox_new (_("Display settings"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), hbox);
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 4);

    /* Autostarted app */
    vbox2 = gtk_vbox_new (FALSE, 2);

    hbox = gtk_hbox_new (FALSE, 0);
    label = gtk_label_new (_("Selected: "));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_size_group_add_widget (size_group, label);
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

    set_autostarted_app_btn ();
    gtk_box_pack_start (GTK_BOX (hbox), dialog->autostarted_btn, TRUE, TRUE, 0);

    gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 2);

    frame = xfce_framebox_new (_("Autostarted application"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), vbox2);
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 4);

    hbox = gtk_hbox_new (FALSE, 0);
    dialog->next_time =
        gtk_check_button_new_with_label (_("Show this dialog at each boot"));
    DEMO_SET_SENSITIVE (dialog->next_time);
    gtk_box_pack_start (GTK_BOX (hbox), dialog->next_time, FALSE, FALSE, 4);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 8);

    /*
     * services tab
     */
    label = gtk_label_new (_("Install"));
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
                              get_service_widget (), label);

    /*
     * Network tab
     */
    vbox = gtk_vbox_new (FALSE, 0);

    label = gtk_label_new (_("Network"));
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);

    vbox2 = gtk_vbox_new (FALSE, 2);

    /* Network settings */
    nm_vbox = gtk_vbox_new (FALSE, 0);
    nm_hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (nm_vbox), nm_hbox, FALSE, FALSE, 2);

    dialog->nm_chk = gtk_check_button_new_with_label (
       _("Enable Network Manager"));
    gtk_box_pack_start (GTK_BOX (nm_hbox), dialog->nm_chk, FALSE, FALSE, 2);

    image = gtk_image_new_from_icon_name ("nm-signal-75", GTK_ICON_SIZE_SMALL_TOOLBAR);
    gtk_box_pack_start (GTK_BOX (nm_hbox), image, FALSE, FALSE, 2);

    nm_is_there = (g_access ("/etc/init/network-manager.conf", R_OK) == 0);
    if (!nm_is_there)
        gtk_widget_set_sensitive (nm_hbox, FALSE);

    net_btn = new_run_cmd_button ("network-wired", _("Configure your network"),
                                  "network-admin", FALSE);
    gtk_box_pack_start (GTK_BOX (nm_vbox), net_btn, FALSE, FALSE, 2);
    DEMO_SET_SENSITIVE (nm_vbox);
    gtk_box_pack_start (GTK_BOX (vbox2), nm_vbox, FALSE, FALSE, 2);

    dialog->mount_dialog = create_mount_gui ();

    shared_folders_btn = new_run_cmd_button ("gnome-fs-share",
                                             _("Shared folders on this Linutop"),
                                             "shares-admin", FALSE);
    gtk_box_pack_start (GTK_BOX (vbox2), shared_folders_btn, FALSE, FALSE, 2);

    win_share_btn = gtk_button_new ();
    img = gtk_image_new_from_icon_name ("gnome-fs-network", GTK_ICON_SIZE_SMALL_TOOLBAR);
    gtk_button_set_image (GTK_BUTTON (win_share_btn), img);
    gtk_button_set_label (GTK_BUTTON (win_share_btn), _("Windows shares access"));
    gtk_button_set_alignment (GTK_BUTTON (win_share_btn), 0, 0.5);
    gtk_box_pack_start (GTK_BOX (vbox2), win_share_btn, FALSE, FALSE, 2);

    frame = xfce_framebox_new (_("Network settings"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);

    xfce_framebox_add (XFCE_FRAMEBOX (frame), vbox2);
    gboolean usable = (gboolean) g_object_get_data (G_OBJECT (net_btn), "usable");
    if (usable && nm_is_there)
        g_signal_connect (G_OBJECT (dialog->nm_chk), "toggled",
            G_CALLBACK (on_nm_check_toggled), (gpointer) net_btn);

    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 4);

    /* vnc */
    vnc_btn = new_run_cmd_button ("gnome-remote-desktop",
                                  _("Configure the remote access service"),
                                  "gx11vnc", TRUE);

    frame = xfce_framebox_new (_("Remote access"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), vnc_btn);
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 4);


    /*
     * advanced tab
     */
    advanced_vbox = gtk_vbox_new (FALSE, 0);

    /* installations */
    backup_btn = new_run_cmd_button ("document-save", _("USB Backup & Restore"),
                                     "lbackup", FALSE);

    frame = xfce_framebox_new (_("System Backup and Restoration"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), backup_btn);
    gtk_box_pack_start (GTK_BOX (advanced_vbox), frame, FALSE, FALSE, 4);
    DEMO_SET_SENSITIVE (backup_btn);

    /* misc */
    vbox2 = gtk_vbox_new (FALSE, 2);

    /* scheduled reboot */
    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 0);
    dialog->reboot_chk = gtk_check_button_new_with_label (
                        _("Schedule a daily reboot at "));
    gtk_box_pack_start (GTK_BOX (hbox), dialog->reboot_chk, FALSE, FALSE, 0);

    dialog->reboot_hour_spin = gtk_spin_button_new_with_range (0, 23, 1);
    label = gtk_label_new (" : ");
    dialog->reboot_min_spin = gtk_spin_button_new_with_range (0, 59, 1);

    gtk_box_pack_start (GTK_BOX (hbox), dialog->reboot_hour_spin, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox), dialog->reboot_min_spin, FALSE, FALSE, 0);
    DEMO_SET_SENSITIVE (hbox);

    dialog->nox_chk =
        gtk_check_button_new_with_label (_
                        ("No graphical interface (command line system)"));
    gtk_box_pack_start (GTK_BOX (vbox2), dialog->nox_chk, FALSE, FALSE, 0);
    DEMO_SET_SENSITIVE (dialog->nox_chk);

    dialog->checkfs_chk =
        gtk_check_button_new_with_label (_
                        ("Check and try to fix filesystem errors"));
    gtk_box_pack_start (GTK_BOX (vbox2), dialog->checkfs_chk, FALSE, FALSE,
               0);
    DEMO_SET_SENSITIVE (dialog->checkfs_chk);

    dialog->nohwclock_chk =
        gtk_check_button_new_with_label (_
                        ("Don't update the hardware clock"));
    gtk_box_pack_start (GTK_BOX (vbox2), dialog->nohwclock_chk, FALSE, FALSE, 0);
    DEMO_SET_SENSITIVE (dialog->nohwclock_chk);

    /* usb cleanup */
    format_btn = new_run_cmd_button ("edit-clear", _("Format a USB key (FAT32)"),
                                     "lusbclean", FALSE);
    gtk_box_pack_start (GTK_BOX (vbox2), format_btn, FALSE, FALSE, 2);

    /* system restoration */
    restore_btn = gtk_button_new_with_label (_("System reset"));
    gtk_button_set_alignment (GTK_BUTTON (restore_btn), 0, 0.5);
    gtk_button_set_image (GTK_BUTTON (restore_btn),
                          gtk_image_new_from_icon_name ("view-refresh",
                                                        GTK_ICON_SIZE_BUTTON));
    gtk_box_pack_start (GTK_BOX (vbox2), restore_btn, FALSE, FALSE, 2);
    g_signal_connect (G_OBJECT (restore_btn), "clicked",
                      G_CALLBACK (on_restore_clicked), NULL);
    DEMO_SET_SENSITIVE (restore_btn);

    frame = xfce_framebox_new (_("Misc"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), vbox2);
    gtk_box_pack_start (GTK_BOX (advanced_vbox), frame, FALSE, FALSE, 4);

    label = gtk_label_new (_("Advanced"));
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), advanced_vbox, label);

    /*
     * security
     */
    security_vbox = gtk_vbox_new (FALSE, 0);

    /* data */
    vbox2 = gtk_vbox_new (FALSE, 2);

    table = gtk_table_new (6, 3, FALSE);
    gtk_table_set_row_spacing (GTK_TABLE (table), 1, 8);
    gtk_table_set_row_spacing (GTK_TABLE (table), 3, 8);
    gtk_box_pack_start (GTK_BOX (vbox2), table, FALSE, FALSE, 0);

    hbox = gtk_hbox_new (FALSE, 0);
    dialog->lock_chk = gtk_radio_button_new (NULL);
    gtk_table_attach (GTK_TABLE (table), dialog->lock_chk, 0, 1, 0, 1,
                      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
    label = gtk_label_new (_("<span weight=\"bold\">"
                             "Linutop Lock (Maximum security)"
                             "</span>"));
    gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_container_add (GTK_CONTAINER (dialog->lock_chk), label);

    label = gtk_label_new (_("- Read only system\n"
                             "- Added data will be discarded after reboot\n"
                             "- Equivalent to a Live CD system\n"
                             "<span weight=\"bold\">"
                             "- Configure your system before switching to this mode!"
                             "</span>"));
    gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
    gtk_table_attach (GTK_TABLE (table), label, 0, 3, 1, 2,
                      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 8, 0);

    hbox = gtk_hbox_new (TRUE, 2);
    dialog->lock_pwd_btn = gtk_button_new_with_label (_("Password"));
    gtk_widget_set_sensitive (dialog->lock_pwd_btn, FALSE);
    gtk_button_set_image (GTK_BUTTON (dialog->lock_pwd_btn),
                          gtk_image_new_from_icon_name ("linutop-lock",
                                                        GTK_ICON_SIZE_BUTTON));
    gtk_box_pack_start (GTK_BOX (hbox), dialog->lock_pwd_btn, TRUE, TRUE, 0);

    dialog->lock_setup_btn = gtk_button_new_with_label (_("Setup"));
    gtk_button_set_image (GTK_BUTTON (dialog->lock_setup_btn),
                          gtk_image_new_from_icon_name ("linutop-lock",
                                                        GTK_ICON_SIZE_BUTTON));
    gtk_box_pack_start (GTK_BOX (hbox), dialog->lock_setup_btn, TRUE, TRUE, 0);

    gtk_table_attach (GTK_TABLE (table), hbox, 2, 3, 0, 1,
                      GTK_SHRINK, GTK_SHRINK, 4, 0);

    hbox = gtk_hbox_new (FALSE, 0);
    dialog->home_chk = gtk_radio_button_new_from_widget (
                        GTK_RADIO_BUTTON (dialog->lock_chk));
    gtk_table_attach (GTK_TABLE (table), dialog->home_chk, 0, 1, 2, 3,
                      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
    label = gtk_label_new (_("<span weight=\"bold\">"
                             "Normal (High security)"
                             "</span>"));
    gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_container_add (GTK_CONTAINER (dialog->home_chk), label);

    label = gtk_label_new (_("- Only user data and settings will be kept after a reboot\n"
                             "- Added applications will be discarded after reboot"));
    gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
    gtk_table_attach (GTK_TABLE (table), label, 0, 3, 3, 4,
                      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 8, 0);

    dialog->full_chk = gtk_radio_button_new_from_widget (
                        GTK_RADIO_BUTTON (dialog->lock_chk));
    gtk_table_attach (GTK_TABLE (table), dialog->full_chk, 0, 1, 4, 5,
                      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
    label = gtk_label_new (_("<span weight=\"bold\">"
                             "Persistent (Medium security)"
                             "</span>"));
    gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_container_add (GTK_CONTAINER (dialog->full_chk), label);

    label = gtk_label_new (_("- All data and settings are kept after reboot\n"
                             "- Added applications are kept after reboot\n"
                             "- Equivalent to a hard drive installation"));
    gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
    gtk_table_attach (GTK_TABLE (table), label, 0, 3, 5, 6,
                      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 8, 0);

    g_signal_connect (G_OBJECT (dialog->lock_chk), "toggled",
             G_CALLBACK (lock_toggled), NULL);
    g_signal_connect (G_OBJECT (dialog->home_chk), "toggled",
             G_CALLBACK (security_toggled), NULL);
    g_signal_connect (G_OBJECT (dialog->full_chk), "toggled",
             G_CALLBACK (security_toggled), NULL);

    frame = xfce_framebox_new (_("System security mode"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), vbox2);
    gtk_box_pack_start (GTK_BOX (security_vbox), frame, FALSE, FALSE, 4);

    label = gtk_label_new (_("Security"));
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), security_vbox, label);
    DEMO_SET_SENSITIVE (security_vbox);

    /* Tools (lock mode) */
    if (dialog->os_info->persistence == LTOP_PERSISTENCE_NONE)
    {
        tools = get_tools_widget ();
        label = gtk_label_new (_("Tools"));
        gtk_notebook_append_page (GTK_NOTEBOOK (notebook), tools, label);
    }

    /* About */
    vbox2 = gtk_vbox_new (FALSE, 0);

    about = get_about_widget ();
    gtk_box_pack_start (GTK_BOX (vbox2), about, FALSE, FALSE, 2);

    hr = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (vbox2), hr, FALSE, FALSE, 2);

    aboutsysinfo = get_info_widget ();
    gtk_box_pack_start (GTK_BOX (vbox2), aboutsysinfo, FALSE, FALSE, 2);

    label = gtk_label_new (_("About"));
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox2, label);

    /* buttons */
    hbox = GTK_DIALOG (dialog->dialog)->action_area;

    dialog->help_btn = gtk_button_new_from_stock (GTK_STOCK_HELP);
    gtk_box_pack_start (GTK_BOX (hbox), dialog->help_btn, TRUE, FALSE, 0);
    gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (hbox),
                                        dialog->help_btn, TRUE);

    dialog->quit_btn = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
    gtk_box_pack_start (GTK_BOX (hbox),dialog->quit_btn, TRUE, FALSE, 0);

#ifndef WITH_DEMO
    dialog->save_btn = gtk_button_new_from_stock (GTK_STOCK_OK);
    gtk_widget_set_sensitive (dialog->save_btn,
                 (g_access (CONFFILE, W_OK) == 0));
#else
    dialog->save_btn = gtk_button_new_from_stock (GTK_STOCK_APPLY);
#endif
    gtk_box_pack_start (GTK_BOX (hbox), dialog->save_btn, TRUE, FALSE, 0);

    read_config ();

    /* signals */
    g_signal_connect (G_OBJECT (dialog->kbd_btn), "clicked",
             G_CALLBACK (kbd_btn_clicked), NULL);
    g_signal_connect (G_OBJECT (dialog->tz_btn), "clicked",
             G_CALLBACK (tz_btn_clicked), NULL);
    g_signal_connect (G_OBJECT (dialog->locale_btn), "clicked",
             G_CALLBACK (locale_btn_clicked), NULL);
    g_signal_connect (G_OBJECT (display_btn), "clicked",
             G_CALLBACK (display_btn_clicked), NULL);

    g_signal_connect (G_OBJECT (win_share_btn), "clicked",
            G_CALLBACK (on_win_share_btn_clicked), NULL);

    g_signal_connect (G_OBJECT (dialog->lock_pwd_btn), "clicked",
             G_CALLBACK (change_pwd), NULL);
    g_signal_connect (G_OBJECT (dialog->lock_setup_btn), "clicked",
             G_CALLBACK (change_lock_setup), NULL);

    g_signal_connect (G_OBJECT (dialog->help_btn), "clicked",
             G_CALLBACK (run_help), NULL);
    g_signal_connect (G_OBJECT (dialog->quit_btn), "clicked",
             G_CALLBACK (delete_event), NULL);
    g_signal_connect (G_OBJECT (dialog->save_btn), "clicked",
             G_CALLBACK (save_config), NULL);
    g_signal_connect (G_OBJECT (dialog->dialog), "delete_event",
             G_CALLBACK (delete_event), NULL);

    gtk_widget_show_all (dialog->dialog);
}

static UniqueResponse
on_present_message_received (UniqueApp *app, gint command,
                             UniqueMessageData *data,
                             guint time_, gpointer user_data)
{
    if (command == 1)
        gtk_window_present (GTK_WINDOW (dialog->dialog));

    return UNIQUE_RESPONSE_OK;
}

int
main (int argc, char *argv[])
{
    gchar *home = NULL;
    gchar *appstarterrc = NULL;
    GFile *path;
    GFileMonitor *monitor;
    GtkWidget *d;
    UniqueApp *u_app;

    xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
    g_thread_init (NULL);
    gdk_threads_init ();
    gtk_init (&argc, &argv);
    gdk_notify_startup_complete ();
    int id = getuid ();
    if (id != 0) {
        d = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
                                GTK_MESSAGE_WARNING,
                                GTK_BUTTONS_OK,
                                _("You don't appear to be root. Exiting."));
        gtk_dialog_run (GTK_DIALOG (d));
        exit (1);
    }

    /* uniqueness of the app */
    u_app = unique_app_new_with_commands ("com.linutop.LConfEditor",
                                          NULL,
                                          "present", 1,
                                          NULL);
    g_signal_connect (G_OBJECT (u_app), "message-received",
                      G_CALLBACK (on_present_message_received), NULL);
    if (unique_app_is_running (u_app))
    {
        unique_app_send_message (u_app, 1, NULL);
        exit (0);
    }

    dsp = XOpenDisplay (NULL);
    g_type_init ();
    engine = xkl_engine_get_instance (dsp);
    registry = xkl_config_registry_get_instance (engine);
    xkl_config_registry_load (registry, FALSE);

    home = get_home ();
    appstarterrc = g_strdup_printf ("%s/.config/appstarterrc", home);
    path = g_file_new_for_path (appstarterrc);
    g_free (appstarterrc);
    g_free (home);

    monitor = g_file_monitor_file (path, 0, NULL, NULL);
    g_signal_connect (G_OBJECT (monitor), "changed",
                      G_CALLBACK (on_appstarterrc_changed), NULL);

    dialog = g_new (gui, 1);
    dialog->lock_pwd = NULL;
    dialog->os_info = ltop_os_info_get ();
    dialog->xorg_config = g_new (xorgconfig, 1);
    dialog->xorg_config->width = 0;
    dialog->xorg_config->height = 0;
    dialog->xorg_config->refresh = 0;

    create_dialog ();

    g_object_set (gtk_settings_get_default (),
                  "gtk-button-images", TRUE, NULL);

    gdk_threads_enter ();
    gtk_main ();
    gdk_threads_leave ();

    g_free (dialog);

    if (have_to_reboot)
        _system (REBOOTCMD);

    return 0;
}
