/**
 * Copyright (C) 2007-2012 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_STIDO_H
#include <stdio.h>
#endif
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <glib/gprintf.h>

#include <gtk/gtk.h>

#include <libxfce4util/libxfce4util.h>

#include "libxfcegui4.h"
#include "ltop-login-box.h"
#include "liblusm.h"

#define MARGIN 2
#define BASE_URL "http://images.linutop.com"

enum {
    COL_ID = 0,
    COL_VERSION,
    COL_VERSION_STRING,
};
enum {
    COL_DEVICE = 0,
    COL_STRING,
};

struct _gui {
    GtkWidget *combo;
    GtkListStore *store;
    GtkWidget *reload_btn;

    GtkListStore *version_store;
    GtkWidget *version_combo;

    GtkWidget *copy_btn;
    GtkWidget *replace_btn;

    GtkWidget *progress_box;
    GtkWidget *pbar;

    GtkWidget *go_btn;
    GtkWidget *quit_btn;
};

static struct _gui *gui;
static gchar *target_uri = NULL;
static gchar *target_id = NULL;
LtopLoginInfo *infos = NULL;

G_LOCK_DEFINE_STATIC (status);
static volatile int status;

#define set_status(val) G_LOCK (status); status=val; G_UNLOCK (status)

static gchar *invoice_option = NULL;
static gchar *email_option = NULL;
static gchar *image_option = NULL;
static gboolean upgrade = FALSE;
static gboolean full_upgrade = FALSE;

static GOptionEntry entries[] = {
    { "invoice", 'o', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_STRING,
      &invoice_option, "Invoice number", "NUMBER" },
    { "email", 'e', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_STRING,
      &email_option, "Email", "EMAIL" },
    { "image", 'c', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_STRING,
      &image_option, "Image code", "CODE" },
    { "upgrade", 'u', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE,
      &upgrade, "Prepare the target to perform an upgrade", NULL },
    { "full-upgrade", 'f', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE,
      &full_upgrade, "Prepare the target to perform a full upgrade", NULL },
    { NULL }
};

static int curl_progress (void *ptr, double dltotal, double dlnow,
                                     double ultotal, double ulnow)
{
    int cur;
    gdouble begin, end;
    static int count = 0;

    /* no need to refresh too much */
    if (count++ < 100)
        return 0;
    count = 0;

    G_LOCK (status);
    cur = status;
    G_UNLOCK (status);
    begin = lusm_status_desc[cur].begin;
    end = lusm_status_desc[cur].end;
    gdk_threads_enter ();
    gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (ptr),
                                   begin + (dlnow / dltotal * (end-begin)));
    gdk_flush ();
    gdk_threads_leave ();

    return 0;
}

static gpointer
duplicate_thread (Device *device)
{
    lusm_disconnect_from_add_remove ();

    set_status (LUSM_STATUS_PREP);
    if (!lusm_partition (device, FALSE))
    {
        set_status (LUSM_STATUS_FAILURE);
        return NULL;
    }

    set_status (LUSM_STATUS_DOWNLOAD_LIVEFS);
    if (!lusm_download_livefs (target_uri, curl_progress, gui->pbar))
    {
        set_status (LUSM_STATUS_FAILURE);
        return NULL;
    }

    set_status (LUSM_STATUS_DOWNLOAD_SQUASHFS);
    if (!lusm_download_squashfs (target_uri, curl_progress, gui->pbar))
    {
        set_status (LUSM_STATUS_FAILURE);
        return NULL;
    }

    set_status (LUSM_STATUS_VERIF);
    if (!lusm_download_check_md5 (target_uri))
    {
        set_status (LUSM_STATUS_FAILURE);
        return NULL;
    }

    set_status (LUSM_STATUS_EXTRA_USER);
    if (!lusm_download_extra (target_uri, curl_progress, gui->pbar))
    {
        set_status (LUSM_STATUS_FAILURE);
        return NULL;
    }

    set_status (LUSM_STATUS_CONFIG);
    if (!lusm_setup (TRUE)) /* don't try to modify extlinux.conf */
    {
        set_status (LUSM_STATUS_FAILURE);
        return NULL;
    }

    set_status (LUSM_STATUS_BOOT);
    if (!lusm_set_bootable (device))
    {
        set_status (LUSM_STATUS_FAILURE);
        return NULL;
    }

    set_status (LUSM_STATUS_SUCCESS);

    ltop_login_box_register_download (target_id,
                                      LTOP_LOGIN_INFO_DATA_TYPE_IMAGE,
                                      email_option, invoice_option);

    return NULL;
}

static gboolean
update_progress (Device *device)
{
    GtkWidget *d;
    static gint old_status = -1;
    static double old_progress = -1;
    gint current_status;
    gboolean ret;

    G_LOCK (status);
    current_status = status;
    G_UNLOCK (status);

    /* nothing new, bye */
    if (old_status == current_status)
        return TRUE;

    gdk_threads_enter ();
    switch (current_status)
    {
    case LUSM_STATUS_SUCCESS:
        gtk_progress_bar_set_text (GTK_PROGRESS_BAR (gui->pbar), _("Done"));
        gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (gui->pbar), 1.0);

        old_status = -1;

        if (upgrade)
        {
            gchar *dev = g_strconcat (device->path, "1", NULL);
            gchar *cmd = g_strconcat (LIBEXECDIR, "/prepare-target-for-upgrade.sh ",
                                      dev, " ",
                                      full_upgrade ? "" : "copy",
                                      NULL);
            system (cmd);
            g_free (dev);
            g_free (cmd);
            system ("reboot");
        }

        d = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
                                    GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
                                    _("Your system has been successfully installed."));
        gtk_dialog_run (GTK_DIALOG (d));
        gtk_widget_destroy (d);
        gtk_widget_hide (gui->pbar);
        gtk_widget_set_sensitive (gui->go_btn, TRUE);
        gtk_widget_set_sensitive (gui->quit_btn, TRUE);
        gtk_widget_set_sensitive (gui->reload_btn, TRUE);

        ret = FALSE;
        break;

    case LUSM_STATUS_FAILURE:
        gtk_progress_bar_set_text (GTK_PROGRESS_BAR (gui->pbar), _("Failure"));
        gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (gui->pbar), 0.0);
        status = old_status; /* keep track of what went wrong */
        old_status = -1;
        d = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
                                    GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
                                    _("An error occured during the installation."));
        gtk_dialog_run (GTK_DIALOG (d));
        gtk_widget_destroy (d);
        gtk_widget_hide (gui->pbar);
        gtk_widget_set_sensitive (gui->go_btn, TRUE);
        gtk_widget_set_sensitive (gui->quit_btn, TRUE);
        gtk_widget_set_sensitive (gui->reload_btn, TRUE);

        ret = FALSE;
        break;

    case LUSM_STATUS_DOWNLOAD_LIVEFS:
    case LUSM_STATUS_DOWNLOAD_SQUASHFS:
    case LUSM_STATUS_EXTRA_USER:
        old_progress = MAX ((gdouble) lusm_status_desc[current_status].begin, old_progress);
        gtk_progress_bar_set_text (GTK_PROGRESS_BAR (gui->pbar),
                                   lusm_status_desc[current_status].msg);
        gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (gui->pbar), old_progress);
        old_status = current_status;

        ret = TRUE;
        break;

    case LUSM_STATUS_PREP:
    case LUSM_STATUS_VERIF:
    case LUSM_STATUS_CONFIG:
    case LUSM_STATUS_BOOT:
    case LUSM_STATUS_SYNC:
        old_progress = MAX ((gdouble) lusm_status_desc[current_status].begin, old_progress);
        gtk_progress_bar_set_text (GTK_PROGRESS_BAR (gui->pbar),
                                   lusm_status_desc[current_status].msg);
        gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (gui->pbar), old_progress);
        old_status = current_status;

        ret = TRUE;
        break;

    default:
        ret = TRUE;
        break;
    }

    gdk_flush ();
    gdk_threads_leave ();

    return ret;
}

static gpointer
start_timer (gpointer data)
{
    while (update_progress (data))
        g_usleep (100000);

    return NULL;
}

static void
run_lusm ()
{
    GtkTreeIter iter;
    GtkTreeModel *model;
    Device *device;
    gchar *version = NULL;
    gchar *id = NULL;

    model = gtk_combo_box_get_model (GTK_COMBO_BOX (gui->combo));
    gtk_combo_box_get_active_iter (GTK_COMBO_BOX (gui->combo), &iter);
    gtk_tree_model_get (model, &iter, COL_DEVICE, &device, -1);
    g_object_unref (model);

    if (! upgrade)
    {
        model = gtk_combo_box_get_model (GTK_COMBO_BOX (gui->version_combo));
        gtk_combo_box_get_active_iter (GTK_COMBO_BOX (gui->version_combo), &iter);
        gtk_tree_model_get (model, &iter, COL_ID, &id, COL_VERSION, &version, -1);
        if (target_uri)
            g_free (target_uri);
        if (target_id)
            g_free (target_id);

        if (g_str_has_prefix (version, "http://"))
            target_uri = g_strdup (version);
        else
            target_uri = g_strdup_printf ("%s/%s", BASE_URL, version);

        target_id = g_strdup (id);

        g_free (id);
        g_free (version);
    }
    else
    {
        target_id = g_strdup ("latest");
        target_uri = g_strdup_printf ("%s/%s", BASE_URL, "latest");
    }

    gtk_widget_show (gui->pbar);
    gtk_widget_set_sensitive (gui->go_btn, FALSE);
    gtk_widget_set_sensitive (gui->quit_btn, FALSE);
    gtk_widget_set_sensitive (gui->reload_btn, FALSE);

    status = 0;

    g_thread_create ((GThreadFunc) duplicate_thread, device, FALSE, NULL);
    g_thread_create ((GThreadFunc) start_timer, device, FALSE, NULL);
}

static void
go_clicked (GtkWidget *btn, gpointer data)
{
    GtkTreeIter iter;
    GtkTreeModel *model;
    GtkWidget *d;
    int response;
    Device *device;
    gchar *str, *msg;

    model = gtk_combo_box_get_model (GTK_COMBO_BOX (gui->combo));
    gtk_combo_box_get_active_iter (GTK_COMBO_BOX (gui->combo), &iter);
    gtk_tree_model_get (model, &iter, COL_DEVICE, &device, COL_STRING, &str, -1);

    if (upgrade)
    {
        msg = g_strdup_printf (_("The system will reboot after the installation.\n"
                                 "Please close all the applications.\n\n"
                                 "The content of %s will be erased.\n"
                                 "Do you really want to continue?"),
                                 str);
    }
    else
    {
        msg = g_strdup_printf (_("The content of %s will be erased.\n"
                                 "Do you really want to continue?"),
                               str);
    }

    d = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
                                GTK_MESSAGE_WARNING, GTK_BUTTONS_YES_NO,
                                "%s", msg);
    response = gtk_dialog_run (GTK_DIALOG (d));
    gtk_widget_destroy (d);

    g_free (msg);
    g_free (str);

    if (response == GTK_RESPONSE_YES)
        run_lusm ();
}

static void
append_to_store (gpointer data, gpointer user_data)
{
    GtkListStore *store = (GtkListStore *) user_data;
    Device *device = (Device *) data;
    GtkTreeIter iter;
    gchar *size;
    gchar *string;

    size = g_format_size_for_display ((goffset) device->size);
    string = g_strdup_printf  ("%s %s (%s)",
                               device->vendor, device->model, size);

    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter,
                       COL_DEVICE, device,
                       COL_STRING, string,
                       -1);

    g_free (size);
    g_free (string);
}

static void
device_free (gpointer data, gpointer user_data)
{
    lusm_device_free ((Device *) data);
}

static void
populate_store (GtkListStore *store)
{
    static GList *devices = NULL;

    gtk_list_store_clear (store);

    if (devices != NULL)
    {
        g_list_foreach (devices, device_free, NULL);
        g_list_free (devices);
        devices = NULL;
    }

    devices = lusm_available_targets ();
    g_list_foreach (devices, (GFunc) append_to_store, store);
}

static void
append_to_version_store (gpointer data, gpointer user_data)
{
    GtkListStore *store = (GtkListStore *) user_data;
    LtopLoginInfoData *info_data = (LtopLoginInfoData *) data;
    GtkTreeIter iter;

    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter,
                        COL_ID, info_data->id,
                        COL_VERSION, info_data->version,
                        COL_VERSION_STRING, info_data->text,
                        -1);
}

static void
populate_version_store (GtkListStore *store)
{
    g_list_foreach (infos->images, (GFunc) append_to_version_store, store);
}

static void
reload_clicked (GtkWidget *btn, gpointer data)
{
    GtkWidget *combo = (GtkWidget *) data;
    GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
    populate_store (GTK_LIST_STORE (model));
    gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
}

static void
on_device_added ()
{
    reload_clicked (NULL, gui->combo);
}

static void
on_full_upgrade_toggled (GtkToggleButton *btn, gpointer data)
{
    full_upgrade = gtk_toggle_button_get_active (btn);
}

static void
validate (GtkWidget *w, GtkWidget *btn)
{
    gboolean sensitive = TRUE;

    if (gtk_combo_box_get_active (GTK_COMBO_BOX (gui->combo)) < 0)
        sensitive = FALSE;

    gtk_widget_set_sensitive (btn, sensitive);
}

static GtkWidget *
build_gui ()
{
    GtkWidget *w, *label;
    GtkWidget *heading;
    GtkWidget *mvbox, *vbox, *hbox, *hr;
    GtkSizeGroup *sgr;
    GtkCellRenderer *renderer;

    w = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (w), _("Linutop Installation"));
    gtk_window_set_resizable (GTK_WINDOW (w), FALSE);
    gtk_window_set_default_icon_name ("internet-web-browser");
    gtk_container_set_border_width (GTK_CONTAINER (w), 6);

    mvbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (w), mvbox);
    gtk_widget_show (mvbox);

    heading = xfce_heading_new ();
    xfce_heading_set_title (XFCE_HEADING (heading),
                            _("Linutop Installation"));
    xfce_heading_set_subtitle (XFCE_HEADING (heading),
                               _("Install a Linutop OS from an Internet location"));
    xfce_heading_set_icon_name (XFCE_HEADING (heading), "internet-web-browser");
    gtk_box_pack_start (GTK_BOX (mvbox), heading, FALSE, FALSE, MARGIN);
    gtk_widget_show (heading);

    sgr = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (mvbox), hbox, FALSE, FALSE, MARGIN);

    label = gtk_label_new (_("Target: "));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_size_group_add_widget (sgr, label);
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, MARGIN);

    gui->store = gtk_list_store_new (2, G_TYPE_POINTER, G_TYPE_STRING);
    gui->combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (gui->store));
    renderer = gtk_cell_renderer_text_new ();
    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (gui->combo), renderer, TRUE);
    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (gui->combo), renderer,
                      "text", COL_STRING);
    gtk_box_pack_start (GTK_BOX (hbox), gui->combo, FALSE, FALSE, MARGIN);

    gui->reload_btn = gtk_button_new_from_stock (GTK_STOCK_REFRESH);
    gtk_widget_set_tooltip_text (gui->reload_btn, _(
            "Click here to detect newly plugged devices"));
    gtk_box_pack_start (GTK_BOX (hbox), gui->reload_btn, FALSE, FALSE, MARGIN);
    g_signal_connect (G_OBJECT (gui->reload_btn), "clicked",
             G_CALLBACK (reload_clicked), gui->combo);

    gtk_widget_show_all (hbox);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (mvbox), hbox, FALSE, FALSE, MARGIN);

    if (upgrade)
    {
        label = gtk_label_new (_("Version: "));
        gtk_misc_set_alignment (GTK_MISC (label), 0, 0.1);
        gtk_size_group_add_widget (sgr, label);
        gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, MARGIN);

        vbox = gtk_vbox_new (FALSE, 0);
        gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, MARGIN);

        gui->copy_btn = gtk_radio_button_new_with_label (NULL,
                _("Only replace essential files (keep personnal data)"));
        gtk_box_pack_start (GTK_BOX (vbox), gui->copy_btn,
                            FALSE, FALSE, 0);

        gui->replace_btn = gtk_radio_button_new_with_label_from_widget (
                GTK_RADIO_BUTTON (gui->copy_btn),
                _("Replace the system entirely (delete everything)"));
        gtk_box_pack_start (GTK_BOX (vbox), gui->replace_btn,
                            FALSE, FALSE, 0);

        g_signal_connect (G_OBJECT (gui->replace_btn), "toggled",
                          G_CALLBACK (on_full_upgrade_toggled), NULL);

        if (full_upgrade)
            gtk_toggle_button_set_active (
                GTK_TOGGLE_BUTTON (gui->replace_btn), TRUE);
        else
            gtk_toggle_button_set_active (
                GTK_TOGGLE_BUTTON (gui->copy_btn), TRUE);
    }
    else
    {
        label = gtk_label_new (_("Version: "));
        gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
        gtk_size_group_add_widget (sgr, label);
        gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, MARGIN);

        gui->version_store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
        gui->version_combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (gui->version_store));
        renderer = gtk_cell_renderer_text_new ();
        gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (gui->version_combo), renderer, TRUE);
        gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (gui->version_combo), renderer,
                            "text", COL_VERSION_STRING);
        gtk_box_pack_start (GTK_BOX (hbox), gui->version_combo, FALSE, FALSE, MARGIN);
    }

    gtk_widget_show_all (hbox);

    gui->pbar = gtk_progress_bar_new ();
    gtk_box_pack_start (GTK_BOX (mvbox), gui->pbar, FALSE, FALSE, MARGIN);

    hr = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (mvbox), hr, FALSE, FALSE, MARGIN);
    gtk_widget_show (hr);

    hbox = gtk_hbox_new (FALSE, MARGIN);
    gtk_box_pack_start (GTK_BOX (mvbox), hbox, FALSE, FALSE, MARGIN);
    gui->go_btn = gtk_button_new_from_stock (GTK_STOCK_APPLY);
    gtk_box_pack_end (GTK_BOX (hbox), gui->go_btn, FALSE, FALSE, MARGIN);
    gui->quit_btn = gtk_button_new_from_stock (GTK_STOCK_QUIT);
    gtk_box_pack_end (GTK_BOX (hbox), gui->quit_btn, FALSE, FALSE, MARGIN);

    gtk_widget_show_all (hbox);

    g_signal_connect (G_OBJECT (w), "destroy", gtk_main_quit, NULL);
    g_signal_connect (G_OBJECT (gui->quit_btn), "clicked", gtk_main_quit, NULL);

    g_signal_connect (G_OBJECT (gui->combo), "changed",
                      G_CALLBACK (validate), gui->go_btn);

    g_signal_connect (G_OBJECT (gui->go_btn), "clicked",
                      G_CALLBACK (go_clicked), NULL);

    return w;
}

void
run_user_cmd (const char *user_cmd, gboolean sn)
{
    uid_t uid = 0;
    char *cuid = NULL;
    gchar cmd[100];
    gchar *run_sn = sn ? "" : "-n";

    cuid = getenv ("SUDO_UID");
    if (cuid)
        uid = atoi (cuid);
    if (uid >= 0) {
        g_sprintf (cmd, "lstart %s -u %d \"%s\"", run_sn, uid, user_cmd);
        xfce_exec (cmd, FALSE, FALSE, NULL);
    }
}

static void
validate_account_no_gui (GtkWidget *w)
{
    GList *p;
    int i = 0;
    GtkWidget *d;

    if (! ltop_login_box_check_connected ())
    {
        d = gtk_message_dialog_new (NULL,
                            GTK_DIALOG_MODAL,
                            GTK_MESSAGE_WARNING,
                            GTK_BUTTONS_OK,
                            _("The service is not accessible.\n\n"
                              "Please check your Internet connection."));
        gtk_dialog_run (GTK_DIALOG (d));
        exit (1);
    }

    infos = ltop_login_box_validate_login (email_option, invoice_option);
    if (!infos->valid)
    {
        d = gtk_message_dialog_new (NULL,
                            GTK_DIALOG_MODAL,
                            GTK_MESSAGE_WARNING,
                            GTK_BUTTONS_OK,
                            _("Your account is not valid."));
        gtk_dialog_run (GTK_DIALOG (d));
        exit (1);
    }

    populate_store (gui->store);
    gtk_combo_box_set_active (GTK_COMBO_BOX (gui->combo), 0);
    if (!upgrade)
    {
        populate_version_store (gui->version_store);
        gtk_combo_box_set_active (GTK_COMBO_BOX (gui->version_combo), 0);

        if (image_option)
        {
            for (p=infos->images; p; p = p->next, i++)
            {
                LtopLoginInfoData *info = (LtopLoginInfoData *)p->data;
                if (strcmp (info->id, image_option) != 0)
                    continue;

                gtk_combo_box_set_active (GTK_COMBO_BOX (gui->version_combo), i);
                break;
            }
        }
    }

    gtk_widget_show (w);
    gtk_main ();
}

static void
on_url_clicked (GtkWidget *w, gpointer data)
{
    run_user_cmd ("firefox http://linutop.com/shop", TRUE);
}

static void
validate_account (GtkWidget *w)
{
    GtkWidget *d;
    GtkWidget *login_box, *label;
    GtkWidget *heading;

    d = gtk_dialog_new_with_buttons (_("Linutop Website Authentication"),
                                     NULL, GTK_DIALOG_MODAL,
                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                     GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
                                     NULL);
    gtk_container_set_border_width (GTK_CONTAINER (d), 6);
    gtk_dialog_set_default_response (GTK_DIALOG (d), GTK_RESPONSE_ACCEPT);

    heading = xfce_heading_new ();
    xfce_heading_set_title (XFCE_HEADING (heading),
                            _("Linutop Website Authentication"));
    xfce_heading_set_icon_name (XFCE_HEADING (heading),
                                "gtk-dialog-authentication");
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d)->vbox), heading, FALSE, FALSE, 0);

    label = sexy_url_label_new ();
    sexy_url_label_set_markup (SEXY_URL_LABEL (label),
        _("<b>To perform an installation</b> please provide:\n"
          " - your invoice number\n"
          " - your email address"));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d)->vbox), label, FALSE, FALSE, 2);

    login_box = ltop_login_box_new (NULL, NULL);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d)->vbox), login_box, FALSE, FALSE, 2);

    label = sexy_url_label_new ();
    sexy_url_label_set_markup (SEXY_URL_LABEL (label),
        _("<b>Purchase an installation</b> on "
          "<a href=\"http://www.linutop.com/shop\">"
          "the Linutop shop</a>."));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d)->vbox), label, FALSE, FALSE, 2);
    g_signal_connect (G_OBJECT (label), "url_activated",
        G_CALLBACK (on_url_clicked), NULL);

    gtk_widget_show_all (GTK_DIALOG (d)->vbox);

    switch (gtk_dialog_run (GTK_DIALOG (d)))
    {
        case GTK_RESPONSE_CANCEL:
        case GTK_RESPONSE_DELETE_EVENT:
            _exit (0);
            break;
        case GTK_RESPONSE_ACCEPT:
            break;
        default:
            infos = NULL;
            goto out;
    }

    if (! ltop_login_box_check_connected ())
    {
        d = gtk_message_dialog_new (NULL,
                            GTK_DIALOG_MODAL,
                            GTK_MESSAGE_WARNING,
                            GTK_BUTTONS_OK,
                            _("The service is not accessible.\n\n"
                              "Please check your Internet connection."));
        gtk_dialog_run (GTK_DIALOG (d));
        exit (1);
    }

    infos = ltop_login_box_login (LTOP_LOGIN_BOX (login_box));

out:
    if (!infos->valid)
    {
        d = gtk_message_dialog_new (NULL,
                            GTK_DIALOG_MODAL,
                            GTK_MESSAGE_WARNING,
                            GTK_BUTTONS_OK,
                            _("Your account is not valid."));
        gtk_dialog_run (GTK_DIALOG (d));
        exit (1);
    }
    else
    {
        email_option = g_strdup (ltop_login_box_get_email (
                                    LTOP_LOGIN_BOX (login_box)));
        invoice_option = g_strdup (ltop_login_box_get_invoice (
                                    LTOP_LOGIN_BOX (login_box)));
        populate_store (gui->store);
        gtk_combo_box_set_active (GTK_COMBO_BOX (gui->combo), 0);
        if (! upgrade)
        {
            populate_version_store (gui->version_store);
            gtk_combo_box_set_active (GTK_COMBO_BOX (gui->version_combo), 0);
        }
        gtk_widget_destroy (d);
        gtk_widget_show (w);
        gtk_main ();
    }
}

int
main (int argc, gchar **argv)
{
    GtkWidget *d;
    GtkWidget *w;

    xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");

    g_thread_init (NULL);
    dbus_g_thread_init ();
    gdk_threads_init ();
    gtk_init_with_args (&argc, &argv, "", entries, NULL, NULL);

    if (full_upgrade)
        upgrade = TRUE;

    if (upgrade)
    {
        g_free (image_option);
        image_option = NULL;
    }

    lusm_init ();
    gui = g_new (struct _gui, 1);
    lusm_connect_to_add_remove (on_device_added, on_device_added);
    w = build_gui ();

    if (getuid () != 0) {
        d = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
                                GTK_MESSAGE_WARNING,
                                GTK_BUTTONS_OK,
                                _("You don't appear to be root. Exiting."));
        gtk_dialog_run (GTK_DIALOG (d));
        exit (1);
    }

    g_object_set (gtk_settings_get_default (),
                  "gtk-button-images", TRUE, NULL);

    gdk_threads_enter ();
    if (invoice_option && email_option)
        validate_account_no_gui (w);
    else
        validate_account (w);
    gdk_threads_leave ();

    lusm_shutdown ();
    g_free (gui);

    return 0;
}
